;******************************************************
; parameter automation, generate curve and write to chn channel
;******************************************************
	instr 2
	Sparameter	strget p4	; the parameter name to create automation for
	puts Sparameter, 1
	iStart		= p5		; start value
	iEnd		= p6		; end value
	iCurve 		= p7		; transeg curve
	print p1, p2, iEnd

	kValue		transeg iStart, p3, iCurve, iEnd
	p3 		= p3 + 2/kr				; modify p3 so the ending value can be reached fully
	Schn		strcat	Sparameter, "Line"
	chnset		kValue, Schn
	endin
;******************************************************

;******************************************************
; parameter automation, generate curve from current value, and write to chn channel
;******************************************************
	instr 3
	Sparameter	strget p4	; the parameter name to create automation for
	puts Sparameter, 1
	iEnd		= p5		; end value    
	iCurve 		= p6            ; transeg curve
	print p1, p2, iEnd
	iStart		chnget	Sparameter
	kValue		transeg iStart, p3, iCurve, iEnd
	p3 		= p3 + 2/kr				; modify p3 so the ending value can be reached fully
	Schn		strcat	Sparameter, "Line"
	chnset		kValue, Schn
	endin
;******************************************************

;******************************************************
; parameter automation, generate LFO and write to chn channel
;******************************************************
	instr 4
	Sparameter	strget p4		; the parameter name to create automation for
	puts Sparameter, 1
	iLfoAmount	= p5			; LFO amount (amp)
	iLfoFreq	= p6			; LFO frequency
	iFadeIn		= (p7 <= 0 ? 2/kr : p7)	; fade must not be zero, or linseg will not work correctly
	iFadeOut	= (p8 <= 0 ? 2/kr : p8)	; fade must not be zero, or linseg will not work correctly
	print p1, p2, iLfoAmount

	; init amp values, to be modified if not a tied note
	iamp0		init	1
	iamp1		init	1
	iamp2		init	1

 	itie     tival                                 ; Check if this note is tied,
	if itie  ==  1     igoto nofadein              ; if not fade in
  	iamp0    init      0
	nofadein:

	if p3    < 0       igoto nofadeout             ; Check if this note is held, if not fade out
  	iamp2    init      0
	nofadeout:

	kEnvelope	linseg	iamp0, iFadeIn, iamp1, p3-(2/kr)-iFadeIn-iFadeOut, iamp1, iFadeOut, iamp2, 1, iamp2	; optional fade in and out of modulator signal

	tigoto tieskip
	kValue		oscil	iLfoAmount, iLfoFreq, giTri	; LFO
	tieskip:

	kValue		= kValue * kEnvelope
	Schn		strcat	Sparameter, "Lfo"
	chnset		kValue, Schn

	endin
;******************************************************

;******************************************************
; parameter automation, generate noise and write to chn channel
;******************************************************
	instr 5
	Sparameter	strget p4			; the parameter name to create automation for
	puts Sparameter, 1
	irangeMin	= p5				; noise minimum value
	irangeMax	= p6				; noise maximum value
	icpsMin		= p7				; minimum frequency for noise (rate of change)
	icpsMax		= p8				; maximum frequency for noise (rate of change)
	iFadeIn		= (p9 <= 0 ? 2/kr : p9)		; fade must not be zero, or linseg will not work correctly
	iFadeOut	= (p10 <= 0 ? 2/kr : p10)	; fade must not be zero, or linseg will not work correctly
	print p1, p2, irangeMax


	; init amp values, to be modified if not a tied note
	iamp0		init	1
	iamp1		init	1
	iamp2		init	1

 	itie     tival                                 ; Check if this note is tied,
	if itie  ==  1     igoto nofadein              ; if not fade in
  	iamp0    init      0
	nofadein:

	if p3    < 0       igoto nofadeout             ; Check if this note is held, if not fade out
  	iamp2    init      0
	nofadeout:

	kEnvelope	linseg	iamp0, iFadeIn, iamp1, p3-(2/kr)-iFadeIn-iFadeOut, iamp1, iFadeOut, iamp2, 1, iamp2	; optional fade in and out of modulator signal

	tigoto tieskip
	kValue 		rspline irangeMin, irangeMax, icpsMin, icpsMax	; generate noise
	tieskip:

	kValue		= kValue * kEnvelope
	Schn		strcat	Sparameter, "Rand"			
	chnset		kValue, Schn
	endin
;******************************************************

;******************************************************
; parameter automation, morph from one ftable to another
; (from excisting table to new one specified by p-fields)
;******************************************************
	instr 10

	; (p6 to p70 sets new table values)
	StableName	strget p4		; ftable name to write to
	puts StableName, 1
	iOriginal	chnget StableName	; find table number
	iCurve		= p5			; transeg curve for morphing transition
	print p1, p2

;generate morph control signal
	kMorph		transeg 	0, p3-(2/kr), iCurve, 1
	p3 		= p3 + 2/kr				; modify p3 so the ending value can be reached fully

; generate new ftable, and a copy of the original
	iTempNew	ftgentmp	0, 0, 64, -2, p6,p7,p8,p9,\
					p10,p11,p12,p13,p14,p15,p16,p17,p18,p19,\
					p20,p21,p22,p23,p24,p25,p26,p27,p28,p29,\
					p30,p31,p32,p33,p34,p35,p36,p37,p38,p39,\
					p40,p41,p42,p43,p44,p45,p46,p47,p48,p49,\
					p50,p51,p52,p53,p54,p55,p56,p57,p58,p59,\
					p60,p61,p62,p63,p64,p65,p66,p67,p68,p69,p70

	iTempOld	ftgentmp	0, 0, 64, -2, 0
			tableicopy	iTempOld, iOriginal
; morph indexing
	iMorphTab	ftgentmp	0, 0, 2, -2, iTempOld, iTempNew

; execute morphing (write new values to original table)
			ftmorf		kMorph, iMorphTab, iOriginal
	endin

;******************************************************
; parameter automation, LFO controls morphing between two ftables 
; (one is the current table, the other one specified by p-fields)
;******************************************************
	instr 11

	; (p9 to p73 sets new table values)
	StableName	strget p4		; ftable name to write to
	puts StableName, 1
	iOriginal	chnget StableName	; find table number
	iLfoAmount	= p5			; LFO amount (amp)
	iLfoFreq	= p6                    ; LFO frequency   
	iFadeIn		= (p7 <= 0 ? 2/kr : p7)	; fade must not be zero, or linseg will not work correctly
	iFadeOut	= (p8 <= 0 ? 2/kr : p8)	; fade must not be zero, or linseg will not work correctly
	print p1, p2, iLfoAmount

	kEnvelope	linseg	0, iFadeIn, 1, p3-(2/kr)-iFadeIn-iFadeOut, 1, iFadeOut, 0, 1, 0	; optional fade in and out of modulator signal
	kMorph		oscil	iLfoAmount*0.5, iLfoFreq, giTri	; LFO
	kMorph		= (kMorph+0.5) * kEnvelope

; generate new ftable, and a copy of the original
	iTempNew	ftgentmp	0, 0, 64, -2, p9,\
					p10,p11,p12,p13,p14,p15,p16,p17,p18,p19,\
					p20,p21,p22,p23,p24,p25,p26,p27,p28,p29,\
					p30,p31,p32,p33,p34,p35,p36,p37,p38,p39,\
					p40,p41,p42,p43,p44,p45,p46,p47,p48,p49,\
					p50,p51,p52,p53,p54,p55,p56,p57,p58,p59,\
					p60,p61,p62,p63,p64,p65,p66,p67,p68,p69,\
					p70,p71,p72,p73

	iTempOld	ftgentmp	0, 0, 64, -2, 0
			tableicopy	iTempOld, iOriginal
; morph indexing
	iMorphTab	ftgentmp	0, 0, 2, -2, iTempOld, iTempNew

; execute morphing (write new values to original table)
			ftmorf		kMorph, iMorphTab, iOriginal
	endin

;******************************************************
; parameter automation, random morphing between two ftables 
; (one is the current table, the other one specified by p-fields)
;******************************************************
	instr 12
	; (p11 to p75 sets new table values)
	StableName	strget p4			; ftable name to write to
	puts StableName, 1
	iOriginal	chnget StableName		; find table number
	irangeMin	= p5				; noise minimum value                         
	irangeMax	= p6                            ; noise maximum value                         
	icpsMin		= p7                            ; minimum frequency for noise (rate of change)
	icpsMax		= p8                            ; maximum frequency for noise (rate of change)
	iFadeIn		= (p9 <= 0 ? 2/kr : p9)		; fade must not be zero, or linseg will not work correctly
	iFadeOut	= (p10 <= 0 ? 2/kr : p10)	; fade must not be zero, or linseg will not work correctly	
	print p1, p2, irangeMax

	kEnvelope	linseg	0, iFadeIn, 1, p3-(2/kr)-iFadeIn-iFadeOut, 1, iFadeOut, 0, 1, 0	; optional fade in and out of modulator signal

	kMorph 		rspline irangeMin, irangeMax, icpsMin, icpsMax	; generate noise
	kMorph		= kMorph * kEnvelope

; generate new ftable, and a copy of the original
	iTempNew	ftgentmp	0, 0, 64, -2, \
					p11,p12,p13,p14,p15,p16,p17,p18,p19,\
					p20,p21,p22,p23,p24,p25,p26,p27,p28,p29,\
					p30,p31,p32,p33,p34,p35,p36,p37,p38,p39,\
					p40,p41,p42,p43,p44,p45,p46,p47,p48,p49,\
					p50,p51,p52,p53,p54,p55,p56,p57,p58,p59,\
					p60,p61,p62,p63,p64,p65,p66,p67,p68,p69,\
					p70,p71,p72,p73,p74,p75

	iTempOld	ftgentmp	0, 0, 64, -2, 0
			tableicopy	iTempOld, iOriginal
; morph indexing
	iMorphTab	ftgentmp	0, 0, 2, -2, iTempOld, iTempNew

; execute morphing (write new values to original table)
			ftmorf		kMorph, iMorphTab, iOriginal
	endin

;******************************************************
; parameter automation, set tables (source waveforms)
;******************************************************
	instr 20

	iVoice		= p4			; partikkel instance
	iWaveSlot	= p5			; wave slot (1,2,3 or 4)
	SwaveSlot	sprintf	"Partikkel%iSourceWave%i", iVoice, iWaveSlot
	SwaveName	strget p6
	puts SwaveName, 1
	print p1, p2

	iTableNum	chnget SwaveName
			chnset	iTableNum, SwaveSlot

	endin
;******************************************************


;******************************************************
; parameter automation, mix values from Line, LFO and noise automation, write to chn channel
;******************************************************
	instr 40

	Sparameter	strget p4			; the parameter name to create automation for
	SchnLine	strcat	Sparameter, "Line"	
	SchnLfo		strcat	Sparameter, "Lfo"
	SchnRand	strcat	Sparameter, "Rand"
	kValLine	chnget	SchnLine		; read line segment automation
	kValLfo		chnget	SchnLfo			; read LFO automation
	kValRand	chnget	SchnRand		; read noise automation
	kValue		= kValLine + kValLfo + kValRand	; mix line, LFO and noise
	;printk2 kValue
			chnset	kValue, Sparameter	; write to parameter chn channel

	endin
;******************************************************

