
;***************************************************
; partikkel instr
;***************************************************

#define PartikkelInstrument(N)#
;***************************************************
; setup of source waveforms
; (needs to be done first, because grain pitch and time pointer depends on source waveform lengths)

; select source waveforms
	kwaveform1	chnget 	"Partikkel$NSourceWave1"	; source audio waveform 1
	kwaveform2	chnget 	"Partikkel$NSourceWave2"	; source audio waveform 2
	kwaveform3	chnget 	"Partikkel$NSourceWave3"	; source audio waveform 3
	kwaveform4	chnget 	"Partikkel$NSourceWave4"	; source audio waveform 4
	
; get source waveform length (used when calculating transposition and time pointer)
	kfilen1		tableng	 kwaveform1		; get length of the first source waveform
	kfilen2		tableng	 kwaveform2		; same as above, for source waveform 2
	kfilen3		tableng	 kwaveform3		; same as above, for source waveform 3
	kfilen4		tableng	 kwaveform4		; same as above, for source waveform 4
	kfildur1	= kfilen1 / sr			; length in seconds, for the first source waveform
	kfildur2	= kfilen2 / sr			; same as above, for source waveform 2
	kfildur3	= kfilen3 / sr			; same as above, for source waveform 3
	kfildur4	= kfilen4 / sr			; same as above, for source waveform 4

; original pitch for each waveform, use if they should be transposed individually
	kwavfreq1	chnget "Partikkel$NGrainPitch1"		; Independent transposition for each source wave
	kwavfreq2	chnget "Partikkel$NGrainPitch2"		; Independent transposition for each source wave
	kwavfreq3	chnget "Partikkel$NGrainPitch3"		; Independent transposition for each source wave
	kwavfreq4	chnget "Partikkel$NGrainPitch4"		; Independent transposition for each source wave

	kwavekey1	= cent(kwavfreq1) / kfildur1
	kwavekey2	= cent(kwavfreq2) / kfildur2
	kwavekey3	= cent(kwavfreq3) / kfildur3
	kwavekey4	= cent(kwavfreq4) / kfildur4

; time pointer (phase). This can be independent for each source waveform.
	ksamplepos1	chnget "Partikkel$NSamplepos1"	; time pointer into source waveform 1
	ksamplepos2	chnget "Partikkel$NSamplepos2"	; time pointer into source waveform 2
	ksamplepos3	chnget "Partikkel$NSamplepos3"	; time pointer into source waveform 3
	ksamplepos4	chnget "Partikkel$NSamplepos4"	; time pointer into source waveform 4
	asamplepos1	interp	ksamplepos1
	asamplepos2	interp	ksamplepos2
	asamplepos3	interp	ksamplepos3
	asamplepos4	interp	ksamplepos4

;***************************************************
; other granular synthesis parameters

; amplitude
	kamp		chnget "Partikkel$NAmp"			; output amplitude

; soft sync
	id$N		= $N
	gapulse$N	init 0
	gaphase$N	init 0
	ksyncGravity	chnget "Partikkel$NSyncGravity"			; sync amount control
	asyncPolarity	limit (int(gaphase$N*2)*2)-1, -1, 1		; use the phase of partikkelsync instance n ($N) to find sync polarity for the sync signal.
									; If the phase of instance n is less than 0.5, we want to nudge it down when synchronizing,
									; and if the phase is > 0.5 we want to nudge it upwards.
	async		= gapulse1*ksyncGravity*asyncPolarity		; prepare sync signal with polarity and strength

	
; grain rate
	kGrainRate	chnget "Partikkel$NGrainRate"		; number of grains per second

; grain rate FM
	kGrFmFreq	chnget "Partikkel$NGrFmFreq"		; FM freq for modulating the grainrate 
	kGrFmIndex	chnget "Partikkel$NGrFmIndex"					; FM index for modulating the grainrate (normally kept in a 0.0 to 1.0 range)
	iGrFmWave	= giSine				; FM waveform, for modulating the grainrate 
	aGrFmSig	oscil kGrFmIndex, kGrFmFreq, iGrFmWave	; audio signal for frequency modulation of grain rate
	agrainfreq	= kGrainRate + (aGrFmSig*kGrainRate)	; add the modulator signal to the grain rate signal

; distribution 
	kdistribution	chnget "Partikkel$NDistribution"		; grain random distribution in time
	idisttab	= gidisttab$N				; probability distribution for random grain masking

; grain shape
	kGrainDur	chnget "Partikkel$NGrainDur"		; length of each grain relative to grain rate 
	kduration	= (kGrainDur*1000)/kGrainRate		; grain dur in milliseconds, relative to grain rate

	ienv_attack	= giSigmoRise 				; grain attack shape (from table)
	ienv_decay	= giSigmoFall 				; grain decay shape (from table)
	ksustain_amount	chnget "Partikkel$NSustain"		; balance between enveloped time(attack+decay) and sustain level time, 0.0 = no time at sustain level
	ka_d_ratio	chnget "Partikkel$NAdRatio"		; balance between attack time and decay time, 0.0 = zero attack time and full decay time

	kenv2amt	chnget "Partikkel$NEnv2Amt"		; amount of secondary enveloping per grain (e.g. for fof synthesis)
	ienv2tab	= giExpFall 				; secondary grain shape (from table), enveloping the whole grain if used

; grain pitch (transpose, or "playback speed")
	kwavfreq	chnget "Partikkel$NGrainPitch"		; (master) transposition (playback speed) of audio inside grains
								; for audio files, original playback speed is achieved at kwavfreq = 1.0
								; for single cycle waveforms, kwavfreq sets the fundamental pitch
								; Independent pitching of the 4 source waveforms can be controlled via
								; "PartikkelGrainPitch1" to "PartikkelGrainPitch4",
								; (see above)

; pitch sweep
	ksweepshape		chnget "Partikkel$NSweepshape"			; grain wave pitch sweep shape (sweep speed), 0.5 is linear sweep
	iwavfreqstarttab 	= giwavfreqstarttab$N				; start freq scalers (ftable), per grain
	iwavfreqendtab		= giwavfreqendtab$N				; end freq scalers (ftable), per grain

; FM of grain pitch (playback speed)
	kPtchFmFreq	chnget "Partikkel$NPitchFmFreq"				; FM freq, modulating waveform pitch
	kPtchFmIndex	chnget "Partikkel$NPitchFmIndex"				; FM index, modulating waveform pitch
	iPtchFmWave	= giSine						; FM waveform, modulating waveform pitch
	ifmamptab	= gifmamptab$N						; FM index scalers (ftable), per grain
	ifmenv		= giTriangleWin 					; FM index envelope, over each grain (from table)
	kPtchFmIndex	= kPtchFmIndex + (kPtchFmIndex*kPtchFmFreq*0.00001) 	; FM index scaling formula
	awavfm		oscil	kPtchFmIndex, kPtchFmFreq, iPtchFmWave		; audio input for frequency modulation inside grain

; trainlet parameters
	icosine		= giCosine					; needs to be a cosine wave to create trainlets
	kTrainCps	chnget "Partikkel$NTrainCps"			; set cps equal to grain freq, creating a single cycle of a trainlet inside each grain
	knumpartials	chnget "Partikkel$NTrainPartials"		; number of partials in trainlet
	kchroma		chnget "Partikkel$NTrainChroma"			; chroma, falloff of partial amplitude towards sr/2

; masking
	igainmasks	= gigainmasks$N					; gain masking table, amplitude for individual grains
	ichannelmasks	= gichannelmasks$N				; channel masking table, output routing for individual grains

	krandommask	chnget "Partikkel$NRandomMask"			; random masking (muting) of individual grains

	; mix of source waves
	iwaveamptab	= giwaveamptab$N				; wave mix masking, gain per source waveform per grain, 
	; xy mixing surface for the 4 source waves, plus separate control for trainlet mix in relation to the other 4 sources
	kWaveX		chnget "Partikkel$NWaveX"
	kWaveY		chnget "Partikkel$NWaveY"
	kTrainAmp	chnget "Partikkel$NTrainAmp"
	; calculate gain for 4 sources from XY position
	kwgain1		limit ((1-kWaveX)*(1-kWaveY)), 0, 1
	kwgain2		limit (kWaveX*(1-kWaveY)), 0, 1
	kwgain3		limit ((1-kWaveX)*kWaveY), 0, 1
	kwgain4		limit (kWaveX*kWaveY), 0, 1
	; modify source wave gains in relation to trainlet amp
	kwavegain1	= kwgain1 * (1-kTrainAmp)	
	kwavegain2	= kwgain2 * (1-kTrainAmp)	
	kwavegain3	= kwgain3 * (1-kTrainAmp)	
	kwavegain4	= kwgain4 * (1-kTrainAmp)	
	; write to wave mix mask table
	tablew		kwavegain1, 2, iwaveamptab	
	tablew		kwavegain2, 3, iwaveamptab
	tablew		kwavegain3, 4, iwaveamptab
	tablew		kwavegain4, 5, iwaveamptab
	tablew		kTrainAmp, 6, iwaveamptab

; system parameter
	imax_grains	= 100				; max number of grains per k-period

;***************************************************
; partikkel opcode

a1,a2,a3,a4,a5,a6,a7,a8 partikkel agrainfreq, kdistribution, idisttab, async, kenv2amt, ienv2tab, \
               	  ienv_attack, ienv_decay, ksustain_amount, ka_d_ratio, kduration, kamp, igainmasks, \
               	  kwavfreq, ksweepshape, iwavfreqstarttab, iwavfreqendtab, awavfm, \
               	  ifmamptab, ifmenv, icosine, kTrainCps, knumpartials, \
               	  kchroma, ichannelmasks, krandommask, kwaveform1, kwaveform2, kwaveform3, kwaveform4, \
               	  iwaveamptab, asamplepos1, asamplepos2, asamplepos3, asamplepos4, \
               	  kwavekey1, kwavekey2, kwavekey3, kwavekey4, imax_grains, id$N

gapulse$N,gaphase$N	partikkelsync	id$N

;***************************************************
; output routing
	chnmix	a1,	"MasterAudioLeft"
	chnmix	a2,	"MasterAudioRight"
	chnmix	a3,	"Ringmod1Send"
	chnmix	a4,	"Bandpass1Send"
	chnmix	a5, 	"DelaySendLeft"
	chnmix	a6, 	"DelaySendRight"
	chnmix	a7, 	"ReverbSendLeft"
	chnmix	a8, 	"ReverbSendRight"

#

;***************************************************

